#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Database v1 Root action implementations"""

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils

from troveclient.i18n import _


def find_instance_or_cluster(database_client_manager,
                             instance_or_cluster):
    """Returns an instance or cluster, found by ID or name,
    along with the type of resource, instance or cluster.
    Raises CommandError if none is found.
    """
    db_instances = database_client_manager.instances

    try:
        return (osc_utils.find_resource(db_instances,
                                        instance_or_cluster),
                'instance')
    except exceptions.CommandError:
        db_clusters = database_client_manager.clusters
        try:
            return (osc_utils.find_resource(db_clusters,
                                            instance_or_cluster),
                    'cluster')
        except exceptions.CommandError:
            raise exceptions.CommandError(
                _("No instance or cluster with a name or ID of '%s' exists.")
                % instance_or_cluster)


class EnableDatabaseRoot(command.ShowOne):

    _description = _("Enables root for an instance and resets "
                     "if already exists.")

    def get_parser(self, prog_name):
        parser = super(EnableDatabaseRoot, self).get_parser(prog_name)
        parser.add_argument(
            'instance_or_cluster',
            metavar='<instance_or_cluster>',
            help=_('ID or name of the instance or cluster.'),
        )
        parser.add_argument(
            '--root_password',
            metavar='<root_password>',
            default=None,
            help=_('Root password to set.'))

        return parser

    def take_action(self, parsed_args):
        database_client_manager = self.app.client_manager.database
        instance_or_cluster, resource_type = find_instance_or_cluster(
            database_client_manager,
            parsed_args.instance_or_cluster)

        db_root = database_client_manager.root
        if resource_type == 'instance':
            root = db_root.create_instance_root(instance_or_cluster,
                                                parsed_args.root_password)
        else:
            root = db_root.create_cluster_root(instance_or_cluster,
                                               parsed_args.root_password)

        result = {'name': root[0],
                  'password': root[1]}
        return zip(*sorted(result.items()))


class DisableDatabaseRoot(command.Command):

    _description = _("Disables root for an instance.")

    def get_parser(self, prog_name):
        parser = super(DisableDatabaseRoot, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.'),
        )

        return parser

    def take_action(self, parsed_args):
        database_client_manager = self.app.client_manager.database

        db_instances = database_client_manager.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)

        db_root = database_client_manager.root
        db_root.disable_instance_root(instance)


class ShowDatabaseRoot(command.ShowOne):

    _description = _("Gets status if root was ever enabled for "
                     "an instance or cluster.")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseRoot, self).get_parser(prog_name)
        parser.add_argument(
            'instance_or_cluster',
            metavar='<instance_or_cluster>',
            help=_('ID or name of the instance or cluster.'),
        )

        return parser

    def take_action(self, parsed_args):
        database_client_manager = self.app.client_manager.database
        instance_or_cluster, resource_type = find_instance_or_cluster(
            database_client_manager,
            parsed_args.instance_or_cluster)

        db_root = database_client_manager.root
        if resource_type == 'instance':
            root = db_root.is_instance_root_enabled(instance_or_cluster)
        else:
            root = db_root.is_cluster_root_enabled(instance_or_cluster)

        result = {'is_root_enabled': root.rootEnabled}
        return zip(*sorted(result.items()))



# Copyright 2016 Tesora, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from troveclient import base


class VolumeType(base.Resource):
    """A VolumeType is an Cinder volume type."""

    def __init__(self, manager, info, loaded=False):
        super(VolumeType, self).__init__(manager, info, loaded)
        if self.id is None and self.str_id is not None:
            self.id = self.str_id

    def __repr__(self):
        return "<VolumeType: %s>" % self.name


class VolumeTypes(base.ManagerWithFind):
    """Manage :class:`VolumeType` resources."""
    resource_class = VolumeType

    def list(self):
        """Get a list of all volume-types.
        :rtype: list of :class:`VolumeType`.
        """
        return self._list("/volume-types", "volume_types")

    def list_datastore_version_associated_volume_types(self, datastore,
                                                       version_id):
        """Get a list of all volume-types for the specified datastore type
        and datastore version .
        :rtype: list of :class:`VolumeType`.
        """
        return self._list("/datastores/%s/versions/%s/volume-types" %
                          (datastore, version_id),
                          "volume_types")

    def get(self, volume_type):
        """Get a specific volume-type.

        :rtype: :class:`VolumeType`
        """
        return self._get("/volume-types/%s" % base.getid(volume_type),
                         "volume_type")



# Copyright 2010 Jacob Kaplan-Moss

# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base utilities to build API operation managers and objects on top of.
"""

import contextlib
import hashlib
import os

from oslo_utils import reflection
from oslo_utils import strutils

from troveclient.compat import exceptions
from troveclient.compat import utils


# Python 2.4 compat
try:
    all
except NameError:
    def all(iterable):
        return True not in (not x for x in iterable)


def getid(obj):
    """Retrives an id from object or integer.

    Abstracts the common pattern of allowing both an object or an object's
    ID as a parameter when dealing with relationships.
    """
    try:
        return obj.id
    except AttributeError:
        return obj


class Manager(utils.HookableMixin):
    """Manager defining CRUD operations for API.

    Managers interact with a particular type of API (servers, flavors,
    images, etc.) and provide CRUD operations for them.
    """
    resource_class = None

    def __init__(self, api):
        self.api = api

    def _list(self, url, response_key, obj_class=None, body=None):
        resp = None
        if body:
            resp, body = self.api.client.post(url, body=body)
        else:
            resp, body = self.api.client.get(url)

        if obj_class is None:
            obj_class = self.resource_class

        data = body[response_key]
        # NOTE(ja): keystone returns values as list as {'values': [ ... ]}
        #           unlike other services which just return the list...
        if isinstance(data, dict):
            try:
                data = data['values']
            except KeyError:
                pass

        with self.completion_cache('human_id', obj_class, mode="w"):
            with self.completion_cache('uuid', obj_class, mode="w"):
                return [obj_class(self, res, loaded=True)
                        for res in data if res]

    @contextlib.contextmanager
    def completion_cache(self, cache_type, obj_class, mode):
        """Bash-completion cache.

        The completion cache store items that can be used for bash
        autocompletion, like UUIDs or human-friendly IDs.

        A resource listing will clear and repopulate the cache.

        A resource create will append to the cache.

        Delete is not handled because listings are assumed to be performed
        often enough to keep the cache reasonably up-to-date.
        """
        base_dir = utils.env('REDDWARFCLIENT_ID_CACHE_DIR',
                             default="~/.troveclient")

        # NOTE(sirp): Keep separate UUID caches for each username + endpoint
        # pair
        username = utils.env('OS_USERNAME', 'USERNAME')
        url = utils.env('OS_URL', 'SERVICE_URL')
        uniqifier = hashlib.md5(username + url).hexdigest()

        cache_dir = os.path.expanduser(os.path.join(base_dir, uniqifier))

        try:
            os.makedirs(cache_dir, 0o755)
        except OSError:
            # NOTE(kiall): This is typically either permission denied while
            #              attempting to create the directory, or the directory
            #              already exists. Either way, don't fail.
            pass

        resource = obj_class.__name__.lower()
        filename = "%s-%s-cache" % (resource, cache_type.replace('_', '-'))
        path = os.path.join(cache_dir, filename)

        cache_attr = "_%s_cache" % cache_type

        try:
            setattr(self, cache_attr, open(path, mode))
        except IOError:
            # NOTE(kiall): This is typically a permission denied while
            #              attempting to write the cache file.
            pass

        try:
            yield
        finally:
            cache = getattr(self, cache_attr, None)
            if cache:
                cache.close()
                delattr(self, cache_attr)

    def write_to_completion_cache(self, cache_type, val):
        cache = getattr(self, "_%s_cache" % cache_type, None)
        if cache:
            cache.write("%s\n" % val)

    def _get(self, url, response_key=None):
        resp, body = self.api.client.get(url)
        if response_key:
            return self.resource_class(self, body[response_key], loaded=True)
        else:
            return self.resource_class(self, body, loaded=True)

    def _create(self, url, body, response_key, return_raw=False, **kwargs):
        self.run_hooks('modify_body_for_create', body, **kwargs)
        resp, body = self.api.client.post(url, body=body)
        if return_raw:
            return body[response_key]

        with self.completion_cache('human_id', self.resource_class, mode="a"):
            with self.completion_cache('uuid', self.resource_class, mode="a"):
                return self.resource_class(self, body[response_key])

    def _delete(self, url):
        resp, body = self.api.client.delete(url)

    def _update(self, url, body, **kwargs):
        self.run_hooks('modify_body_for_update', body, **kwargs)
        resp, body = self.api.client.put(url, body=body)
        return body


class ManagerWithFind(Manager):
    """Like a `Manager`, but with additional `find()`/`findall()` methods."""

    def find(self, **kwargs):
        """Find a single item with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        matches = self.findall(**kwargs)
        num_matches = len(matches)
        if num_matches == 0:
            msg = "No %s matching %s." % (self.resource_class.__name__, kwargs)
            raise exceptions.NotFound(404, msg)
        elif num_matches > 1:
            raise exceptions.NoUniqueMatch
        else:
            return matches[0]

    def findall(self, **kwargs):
        """Find all items with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        found = []
        searches = kwargs.items()

        for obj in self.list():
            try:
                if all(getattr(obj, attr) == value
                       for (attr, value) in searches):
                    found.append(obj)
            except AttributeError:
                continue

        return found

    def list(self):
        raise NotImplementedError


class Resource(object):
    """A resource represents a particular instance of an object like server.

    This is pretty much just a bag for attributes.
    :param manager: Manager object
    :param info: dictionary representing resource attributes
    :param loaded: prevent lazy-loading if set to True
    """
    HUMAN_ID = False

    def __init__(self, manager, info, loaded=False):
        self.manager = manager
        self._info = info
        self._add_details(info)
        self._loaded = loaded

        # NOTE(sirp): ensure `id` is already present because if it isn't we'll
        # enter an infinite loop of __getattr__ -> get -> __init__ ->
        # __getattr__ -> ...
        if 'id' in self.__dict__ and len(str(self.id)) == 36:
            self.manager.write_to_completion_cache('uuid', self.id)

        human_id = self.human_id
        if human_id:
            self.manager.write_to_completion_cache('human_id', human_id)

    @property
    def human_id(self):
        """Provides a pretty ID which can be used for bash completion."""
        if 'name' in self.__dict__ and self.HUMAN_ID:
            return strutils.to_slug(self.name)
        return None

    def _add_details(self, info):
        for (k, v) in info.items():
            try:
                setattr(self, k, v)
            except AttributeError:
                # In this case we already defined the attribute on the class
                pass

    def __getattr__(self, k):
        if k not in self.__dict__:
            # NOTE(bcwaldon): disallow lazy-loading if already loaded once
            if not self.is_loaded():
                self.get()
                return self.__getattr__(k)

            raise AttributeError(k)
        else:
            return self.__dict__[k]

    def __repr__(self):
        reprkeys = sorted(k for k in self.__dict__.keys()
                          if k[0] != '_' and k != 'manager')
        info = ", ".join("%s=%s" % (k, getattr(self, k)) for k in reprkeys)
        self_cls_name = reflection.get_class_name(self,
                                                  fully_qualified=False)
        return "<%s %s>" % (self_cls_name, info)

    def get(self):
        # set_loaded() first ... so if we have to bail, we know we tried.
        self.set_loaded(True)
        if not hasattr(self.manager, 'get'):
            return

        new = self.manager.get(self.id)
        if new:
            self._add_details(new._info)

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        if hasattr(self, 'id') and hasattr(other, 'id'):
            return self.id == other.id
        return self._info == other._info

    def is_loaded(self):
        return self._loaded

    def set_loaded(self, val):
        self._loaded = val



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import inspect
import types
from unittest import mock

import pkg_resources
import testtools

import troveclient.shell


class DiscoverTest(testtools.TestCase):

    def test_discover_via_entry_points(self):

        def mock_iter_entry_points(group):
            if group == 'troveclient.extension':
                fake_ep = mock.Mock()
                fake_ep.name = 'foo'
                fake_ep.module = types.ModuleType('foo')
                fake_ep.load.return_value = fake_ep.module
                return [fake_ep]

        @mock.patch.object(pkg_resources, 'iter_entry_points',
                           mock_iter_entry_points)
        def test():
            shell = troveclient.shell.OpenStackTroveShell()
            for name, module in shell._discover_via_entry_points():
                self.assertEqual('foo', name)
                self.assertTrue(inspect.ismodule(module))

        test()

    def test_discover_extensions(self):

        def mock_discover_via_python_path(self):
            yield 'foo', types.ModuleType('foo')

        def mock_discover_via_contrib_path(self, version):
            yield 'bar', types.ModuleType('bar')

        def mock_discover_via_entry_points(self):
            yield 'baz', types.ModuleType('baz')

        @mock.patch.object(troveclient.shell.OpenStackTroveShell,
                           '_discover_via_python_path',
                           mock_discover_via_python_path)
        @mock.patch.object(troveclient.shell.OpenStackTroveShell,
                           '_discover_via_contrib_path',
                           mock_discover_via_contrib_path)
        @mock.patch.object(troveclient.shell.OpenStackTroveShell,
                           '_discover_via_entry_points',
                           mock_discover_via_entry_points)
        def test():
            shell = troveclient.shell.OpenStackTroveShell()
            extensions = shell._discover_extensions('1.0')
            self.assertEqual(3, len(extensions))
            names = sorted(['foo', 'bar', 'baz'])
            sorted_extensions = sorted(extensions, key=lambda ext: ext.name)
            for i in range(len(names)):
                ext = sorted_extensions[i]
                name = names[i]
                self.assertEqual(name, ext.name)
                self.assertTrue(inspect.ismodule(ext.module))

        test()



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from unittest import mock

from osc_lib import exceptions
from osc_lib import utils
from oslo_utils import uuidutils

from troveclient import common
from troveclient.osc.v1 import database_backups
from troveclient.tests.osc.v1 import fakes


class TestBackups(fakes.TestDatabasev1):
    fake_backups = fakes.FakeBackups()

    def setUp(self):
        super(TestBackups, self).setUp()
        self.mock_client = self.app.client_manager.database
        self.backup_client = self.app.client_manager.database.backups
        self.instance_client = self.app.client_manager.database.instances


class TestBackupList(TestBackups):

    columns = database_backups.ListDatabaseBackups.columns
    values = ('bk-1234', '1234', 'bkp_1', 'COMPLETED', None,
              '2015-05-16T14:23:08', '262db161-d3e4-4218-8bde-5bd879fc3e61')

    def setUp(self):
        super(TestBackupList, self).setUp()
        self.cmd = database_backups.ListDatabaseBackups(self.app, None)
        data = [self.fake_backups.get_backup_bk_1234()]
        self.backup_client.list.return_value = common.Paginated(data)

    def test_backup_list_defaults(self):
        parsed_args = self.check_parser(self.cmd, [], [])
        columns, data = self.cmd.take_action(parsed_args)

        params = {
            'datastore': None,
            'limit': None,
            'marker': None,
            'instance_id': None,
            'all_projects': False,
            'project_id': None
        }

        self.backup_client.list.assert_called_once_with(**params)
        self.assertEqual(self.columns, columns)
        self.assertEqual([self.values], data)

    def test_backup_list_by_instance_id(self):
        parsed_args = self.check_parser(self.cmd, ["--instance-id", "fake_id"],
                                        [])
        self.cmd.take_action(parsed_args)

        params = {
            'datastore': None,
            'limit': None,
            'marker': None,
            'instance_id': 'fake_id',
            'all_projects': False,
            'project_id': None
        }

        self.backup_client.list.assert_called_once_with(**params)

    def test_backup_list_all_projects(self):
        parsed_args = self.check_parser(self.cmd, ["--all-projects"], [])
        self.cmd.take_action(parsed_args)

        params = {
            'datastore': None,
            'limit': None,
            'marker': None,
            'instance_id': None,
            'all_projects': True,
            'project_id': None
        }

        self.backup_client.list.assert_called_once_with(**params)

    def test_backup_list_by_project(self):
        parsed_args = self.check_parser(self.cmd, ["--project-id", "fake_id"],
                                        [])
        self.cmd.take_action(parsed_args)

        params = {
            'datastore': None,
            'limit': None,
            'marker': None,
            'instance_id': None,
            'all_projects': False,
            'project_id': 'fake_id'
        }

        self.backup_client.list.assert_called_once_with(**params)


class TestBackupListInstance(TestBackups):

    defaults = {
        'limit': None,
        'marker': None
    }

    columns = database_backups.ListDatabaseInstanceBackups.columns
    values = ('bk-1234', '1234', 'bkp_1', 'COMPLETED', None,
              '2015-05-16T14:23:08')

    def setUp(self):
        super(TestBackupListInstance, self).setUp()
        self.cmd = database_backups.ListDatabaseInstanceBackups(self.app, None)
        data = [self.fake_backups.get_backup_bk_1234()]
        self.instance_client.backups.return_value = common.Paginated(data)

    @mock.patch.object(utils, 'find_resource')
    def test_backup_list_defaults(self, mock_find):
        args = ['1234']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.instance_client.backups.assert_called_once_with('1234',
                                                             **self.defaults)
        self.assertEqual(self.columns, columns)
        self.assertEqual([self.values], data)


class TestBackupShow(TestBackups):

    values = ('2015-05-16T14:22:28', 'mysql', '5.6', 'v-56', None, 'bk-1234',
              '1234',
              'http://backup_srvr/database_backups/bk-1234.xbstream.gz.enc',
              'bkp_1', None,
              '262db161-d3e4-4218-8bde-5bd879fc3e61',
              0.11, 'COMPLETED', '2015-05-16T14:23:08')

    def setUp(self):
        super(TestBackupShow, self).setUp()
        self.cmd = database_backups.ShowDatabaseBackup(self.app, None)
        self.data = self.fake_backups.get_backup_bk_1234()
        self.backup_client.get.return_value = self.data
        self.columns = (
            'created',
            'datastore',
            'datastore_version',
            'datastore_version_id',
            'description',
            'id',
            'instance_id',
            'locationRef',
            'name',
            'parent_id',
            'project_id',
            'size',
            'status',
            'updated',
        )

    def test_show(self):
        args = ['bkp_1']
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.values, data)


class TestDatabaseBackupDelete(TestBackups):

    def setUp(self):
        super(TestDatabaseBackupDelete, self).setUp()
        self.cmd = database_backups.DeleteDatabaseBackup(self.app, None)

    @mock.patch("troveclient.utils.get_resource_id_by_name")
    def test_backup_delete(self, mock_getid):
        args = ['backup1']
        mock_getid.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)
        self.backup_client.delete.assert_called_with('backup1')

    @mock.patch("troveclient.utils.get_resource_id_by_name")
    def test_backup_delete_with_exception(self, mock_getid):
        args = ['fakebackup']
        parsed_args = self.check_parser(self.cmd, args, [])

        mock_getid.side_effect = exceptions.CommandError
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)

    @mock.patch("troveclient.utils.get_resource_id_by_name")
    def test_backup_bulk_delete(self, mock_getid):
        backup_1 = uuidutils.generate_uuid()
        backup_2 = uuidutils.generate_uuid()
        mock_getid.return_value = backup_1

        args = ["fake_backup", backup_2]
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        mock_getid.assert_called_once_with(self.backup_client, "fake_backup")
        calls = [mock.call(backup_1), mock.call(backup_2)]
        self.backup_client.delete.assert_has_calls(calls)


class TestBackupCreate(TestBackups):

    values = ('2015-05-16T14:22:28', 'mysql', '5.6', 'v-56', None, 'bk-1234',
              '1234',
              'http://backup_srvr/database_backups/bk-1234.xbstream.gz.enc',
              'bkp_1', None,
              '262db161-d3e4-4218-8bde-5bd879fc3e61',
              0.11, 'COMPLETED', '2015-05-16T14:23:08')

    def setUp(self):
        super(TestBackupCreate, self).setUp()
        self.cmd = database_backups.CreateDatabaseBackup(self.app, None)
        self.data = self.fake_backups.get_backup_bk_1234()
        self.backup_client.create.return_value = self.data
        self.columns = (
            'created',
            'datastore',
            'datastore_version',
            'datastore_version_id',
            'description',
            'id',
            'instance_id',
            'locationRef',
            'name',
            'parent_id',
            'project_id',
            'size',
            'status',
            'updated',
        )

    def test_backup_create_return_value(self):
        args = ['bk-1234', '--instance', self.random_uuid()]
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.values, data)

    @mock.patch('troveclient.utils.get_resource_id_by_name')
    def test_backup_create(self, mock_find):
        args = ['bk-1234-1', '--instance', '1234']
        mock_find.return_value = 'fake-instance-id'
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)
        self.backup_client.create.assert_called_with('bk-1234-1',
                                                     'fake-instance-id',
                                                     description=None,
                                                     parent_id=None,
                                                     incremental=False,
                                                     swift_container=None)

    @mock.patch('troveclient.utils.get_resource_id_by_name')
    def test_incremental_backup_create(self, mock_find):
        args = ['bk-1234-2', '--instance', '1234', '--description',
                'backup 1234', '--parent', '1234-1', '--incremental']
        mock_find.return_value = 'fake-instance-id'

        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        self.backup_client.create.assert_called_with('bk-1234-2',
                                                     'fake-instance-id',
                                                     description='backup 1234',
                                                     parent_id='1234-1',
                                                     incremental=True,
                                                     swift_container=None)

    def test_create_from_data_location(self):
        name = self.random_name('backup')
        ds_version = self.random_uuid()
        args = [name, '--restore-from', 'fake-remote-location',
                '--restore-datastore-version', ds_version, '--restore-size',
                '3']
        parsed_args = self.check_parser(self.cmd, args, [])

        self.cmd.take_action(parsed_args)

        self.backup_client.create.assert_called_with(
            name,
            None,
            restore_from='fake-remote-location',
            restore_ds_version=ds_version,
            restore_size=3,
        )

    def test_required_params_missing(self):
        args = [self.random_name('backup')]
        parsed_args = self.check_parser(self.cmd, args, [])
        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestDatabaseBackupExecutionDelete(TestBackups):

    def setUp(self):
        super(TestDatabaseBackupExecutionDelete, self).setUp()
        self.cmd = database_backups.DeleteDatabaseBackupExecution(
            self.app, None)

    def test_execution_delete(self):
        args = ['execution']
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.backup_client.execution_delete.assert_called_with('execution')
        self.assertIsNone(result)



# Copyright 2020 Catalyst Cloud
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from troveclient.osc.v1 import database_backup_strategy
from troveclient.tests.osc.v1 import fakes
from troveclient.v1 import backup_strategy


class TestBackupStrategy(fakes.TestDatabasev1):
    def setUp(self):
        super(TestBackupStrategy, self).setUp()
        self.manager = self.app.client_manager.database.backup_strategies


class TestBackupStrategyList(TestBackupStrategy):
    def setUp(self):
        super(TestBackupStrategyList, self).setUp()
        self.cmd = database_backup_strategy.ListDatabaseBackupStrategies(
            self.app, None)

    def test_list(self):
        item = backup_strategy.BackupStrategy(
            None,
            {
                'project_id': 'fake_project_id',
                'instance_id': 'fake_instance_id',
                'swift_container': 'fake_container'
            }
        )
        self.manager.list.return_value = [item]

        parsed_args = self.check_parser(self.cmd, [], [])
        columns, data = self.cmd.take_action(parsed_args)

        self.manager.list.assert_called_once_with(instance_id=None,
                                                  project_id=None)
        self.assertEqual(
            database_backup_strategy.ListDatabaseBackupStrategies.columns,
            columns)
        self.assertEqual(
            [('fake_project_id', 'fake_instance_id', 'fake_container')],
            data)


class TestBackupStrategyCreate(TestBackupStrategy):
    def setUp(self):
        super(TestBackupStrategyCreate, self).setUp()
        self.cmd = database_backup_strategy.CreateDatabaseBackupStrategy(
            self.app, None)

    def test_create(self):
        args = ['--instance-id', 'fake_instance_id', '--swift-container',
                'fake_container']
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        self.manager.create.assert_called_once_with(
            instance_id='fake_instance_id',
            swift_container='fake_container'
        )


class TestBackupStrategyDelete(TestBackupStrategy):
    def setUp(self):
        super(TestBackupStrategyDelete, self).setUp()
        self.cmd = database_backup_strategy.DeleteDatabaseBackupStrategy(
            self.app, None)

    def test_delete(self):
        args = ['--instance-id', 'fake_instance_id', '--project-id',
                'fake_project']
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        self.manager.delete.assert_called_once_with(
            project_id='fake_project',
            instance_id='fake_instance_id',
        )



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from troveclient.v1 import users

"""
Unit tests for users.py
"""


class UserTest(testtools.TestCase):
    def setUp(self):
        super(UserTest, self).setUp()
        self.orig__init = users.User.__init__
        users.User.__init__ = mock.Mock(return_value=None)
        self.user = users.User()

    def tearDown(self):
        super(UserTest, self).tearDown()
        users.User.__init__ = self.orig__init

    def test___repr__(self):
        self.user.name = "user-1"
        self.assertEqual('<User: user-1>', self.user.__repr__())


class UsersTest(testtools.TestCase):
    def setUp(self):
        super(UsersTest, self).setUp()
        self.orig__init = users.Users.__init__
        users.Users.__init__ = mock.Mock(return_value=None)
        self.users = users.Users()
        self.users.api = mock.Mock()
        self.users.api.client = mock.Mock()

        self.instance_with_id = mock.Mock()
        self.instance_with_id.id = 215

    def tearDown(self):
        super(UsersTest, self).tearDown()
        users.Users.__init__ = self.orig__init

    def _get_mock_method(self):
        self._resp = mock.Mock()
        self._body = None
        self._url = None

        def side_effect_func(url, body=None):
            self._body = body
            self._url = url
            return (self._resp, body)

        return mock.Mock(side_effect=side_effect_func)

    def _build_fake_user(self, name, hostname=None, password=None,
                         databases=None):
        return {
            'name': name,
            'password': password if password else 'password',
            'host': hostname,
            'databases': databases if databases else [],
        }

    def test_create(self):
        self.users.api.client.post = self._get_mock_method()
        self._resp.status_code = 200
        user = self._build_fake_user('user1')

        self.users.create(23, [user])
        self.assertEqual('/instances/23/users', self._url)
        self.assertEqual({"users": [user]}, self._body)

        # Even if host isn't supplied originally,
        # the default is supplied.
        del user['host']
        self.users.create(23, [user])
        self.assertEqual('/instances/23/users', self._url)
        user['host'] = '%'
        self.assertEqual({"users": [user]}, self._body)

        # If host is supplied, of course it's put into the body.
        user['host'] = '127.0.0.1'
        self.users.create(23, [user])
        self.assertEqual({"users": [user]}, self._body)

        # test creation with the instance as an object
        self.users.create(self.instance_with_id, [user])
        self.assertEqual({"users": [user]}, self._body)

        # Make sure that response of 400 is recognized as an error.
        user['host'] = '%'
        self._resp.status_code = 400
        self.assertRaises(Exception, self.users.create, 12, [user])

    def test_delete(self):
        self.users.api.client.delete = self._get_mock_method()
        self._resp.status_code = 200
        self.users.delete(27, 'user1')
        self.assertEqual('/instances/27/users/user1', self._url)
        self.users.delete(self.instance_with_id, 'user1')
        self.assertEqual('/instances/%s/users/user1' %
                         self.instance_with_id.id, self._url)
        self._resp.status_code = 400
        self.assertRaises(Exception, self.users.delete, 34, 'user1')

    def test_list(self):
        page_mock = mock.Mock()
        self.users._paginated = page_mock
        self.users.list('instance1')
        page_mock.assert_called_with('/instances/instance1/users',
                                     'users', None, None)
        limit = 'test-limit'
        marker = 'test-marker'
        self.users.list('instance1', limit, marker)
        page_mock.assert_called_with('/instances/instance1/users',
                                     'users', limit, marker)

    def test_update_no_changes(self):
        self.users.api.client.post = self._get_mock_method()
        self._resp.status_code = 200
        username = 'upd_user'
        user = self._build_fake_user(username)

        self.users.create(99, [user])
        instance = 'instance1'
        newuserattr = None
        self.assertRaises(Exception, self.users.update_attributes, instance,
                          username, newuserattr)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from osc_lib.command import command
from osc_lib import utils as osc_utils

from troveclient import exceptions
from troveclient.i18n import _


class ListDatabaseLogs(command.Lister):

    _description = _("Lists the log files available for instance.")
    columns = ['Name', 'Type', 'Status', 'Published', 'Pending',
               'Container', 'Prefix']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseLogs, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.')
        )
        return parser

    def take_action(self, parsed_args):
        database_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(database_instances,
                                           parsed_args.instance)
        log_list = database_instances.log_list(instance)
        logs = [osc_utils.get_item_properties(l, self.columns)
                for l in log_list]
        return self.columns, logs


class SetDatabaseInstanceLog(command.ShowOne):
    _description = _("Instructs Trove guest to operate logs.")

    def get_parser(self, prog_name):
        parser = super(SetDatabaseInstanceLog, self).get_parser(prog_name)

        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('Id or Name of the instance.')
        )
        parser.add_argument(
            'log_name',
            metavar='<log_name>',
            type=str,
            help=_('Name of log to operate.')
        )
        parser.add_argument(
            '--enable',
            action='store_true',
            help="Whether or not to enable log collection.",
        )
        parser.add_argument(
            '--disable',
            action='store_true',
            help="Whether or not to disable log collection.",
        )
        parser.add_argument(
            '--publish',
            action='store_true',
            help="Whether or not to publish log files to the backend storage "
                 "for logs(Swift by default).",
        )
        parser.add_argument(
            '--discard',
            action='store_true',
            help="Whether or not to discard the existing logs before publish.",
        )

        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)

        log_info = db_instances.log_action(
            instance, parsed_args.log_name,
            enable=parsed_args.enable,
            disable=parsed_args.disable,
            discard=parsed_args.discard,
            publish=parsed_args.publish
        )
        result = log_info._info

        return zip(*sorted(result.items()))


class ShowDatabaseInstanceLog(command.ShowOne):
    _description = _("Show information of given log name for the database "
                     "instance.")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseInstanceLog, self).get_parser(prog_name)

        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('Id or Name of the instance.')
        )
        parser.add_argument(
            'log_name',
            metavar='<log_name>',
            type=str,
            help=_('Name of log to operate.')
        )

        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)

        log_info = db_instances.log_show(instance, parsed_args.log_name)
        result = log_info._info

        return zip(*sorted(result.items()))


class ShowDatabaseInstanceLogContents(command.Command):
    _description = _("Show the content of log file.")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseInstanceLogContents, self).get_parser(
            prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('Id or Name of the instance.')
        )
        parser.add_argument(
            'log_name',
            metavar='<log_name>',
            type=str,
            help=_('Name of log to operate.')
        )
        parser.add_argument(
            '--lines', default=50, type=int,
            help="The number of log lines can be shown in batch.",
        )

        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)

        try:
            log_gen = db_instances.log_generator(instance,
                                                 parsed_args.log_name,
                                                 lines=parsed_args.lines)
            for log_part in log_gen():
                print(log_part, end="")
        except exceptions.GuestLogNotFoundError:
            print(
                "ERROR: No published '%(log_name)s' log was found for "
                "%(instance)s" % {'log_name': parsed_args.log_name,
                                  'instance': instance}
            )
        except Exception as ex:
            error_msg = ex.message.split('\n')
            print(error_msg[0])


class SaveDatabaseInstanceLog(command.Command):
    _description = _("Save the log file.")

    def get_parser(self, prog_name):
        parser = super(SaveDatabaseInstanceLog, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('Id or Name of the instance.')
        )
        parser.add_argument(
            'log_name',
            metavar='<log_name>',
            type=str,
            help=_('Name of log to operate.')
        )
        parser.add_argument(
            '--file',
            help="Path of file to save log to for instance.",
        )

        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)

        try:
            filepath = db_instances.log_save(instance,
                                             parsed_args.log_name,
                                             filename=parsed_args.file)
            print(_('Log "%(log_name)s" written to %(file_name)s')
                  % {'log_name': parsed_args.log_name,
                     'file_name': filepath})
        except exceptions.GuestLogNotFoundError:
            print(
                "ERROR: No published '%(log_name)s' log was found for "
                "%(instance)s" % {'log_name': parsed_args.log_name,
                                  'instance': instance}
            )
        except Exception as ex:
            error_msg = ex.message.split('\n')
            print(error_msg[0])



# Copyright (c) 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from troveclient.compat import base


class Version(base.Resource):
    """Version is an opaque instance used to hold version information."""
    def __repr__(self):
        return "<Version: %s>" % self.id


class Versions(base.ManagerWithFind):
    """Manage :class:`Versions` information."""

    resource_class = Version

    def index(self, url):
        """Get a list of all versions.

        :rtype: list of :class:`Versions`.
        """
        resp, body = self.api.client.request(url, "GET")
        return [self.resource_class(self, res) for res in body['versions']]



# Copyright 2020 Catalyst Cloud
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from osc_lib.command import command
from osc_lib import utils as osc_utils

from troveclient.i18n import _


class ListDatabaseBackupStrategies(command.Lister):
    _description = _("List backup strategies")
    columns = ['Project ID', 'Instance ID', 'Swift Container']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseBackupStrategies, self).get_parser(
            prog_name)

        parser.add_argument(
            '--instance-id',
            help=_('Filter results by database instance ID.')
        )
        parser.add_argument(
            '--project-id',
            help=_('Project ID in Keystone. Only admin user is allowed to '
                   'list backup strategy for other projects.')
        )

        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database.backup_strategies
        result = manager.list(instance_id=parsed_args.instance_id,
                              project_id=parsed_args.project_id)
        backup_strategies = [osc_utils.get_item_properties(item, self.columns)
                             for item in result]

        return self.columns, backup_strategies


class CreateDatabaseBackupStrategy(command.ShowOne):
    _description = _("Creates backup strategy for the project or a particular "
                     "instance.")

    def get_parser(self, prog_name):
        parser = super(CreateDatabaseBackupStrategy, self).get_parser(
            prog_name)

        parser.add_argument(
            '--project-id',
            help=_('Project ID in Keystone. Only admin user is allowed to '
                   'create backup strategy for other projects.')
        )
        parser.add_argument(
            '--instance-id',
            help=_('Database instance ID.')
        )
        parser.add_argument(
            '--swift-container',
            help=_('The container name for storing the backup data when Swift '
                   'is used as backup storage backend.')
        )

        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database.backup_strategies
        result = manager.create(
            instance_id=parsed_args.instance_id,
            swift_container=parsed_args.swift_container
        )
        return zip(*sorted(result.to_dict().items()))


class DeleteDatabaseBackupStrategy(command.Command):
    _description = _("Deletes backup strategy.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatabaseBackupStrategy, self).get_parser(
            prog_name)
        parser.add_argument(
            '--project-id',
            help=_('Project ID in Keystone. Only admin user is allowed to '
                   'delete backup strategy for other projects.')
        )
        parser.add_argument(
            '--instance-id',
            help=_('Database instance ID.')
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database.backup_strategies
        manager.delete(instance_id=parsed_args.instance_id,
                       project_id=parsed_args.project_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Database v1 Instances action implementations"""

import argparse

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_utils import uuidutils

from troveclient import exceptions
from troveclient.i18n import _
from troveclient.osc.v1 import base
from troveclient import utils as trove_utils


def get_instances_info(instances):
    instances_info = []

    for instance in instances:
        # To avoid invoking GET request to trove.
        instance_info = instance.to_dict()

        instance_info['flavor_id'] = instance.flavor['id']

        instance_info['size'] = '-'
        if 'volume' in instance_info:
            instance_info['size'] = instance_info['volume']['size']

        instance_info['role'] = ''
        if 'replica_of' in instance_info:
            instance_info['role'] = 'replica'
        if 'replicas' in instance_info:
            instance_info['role'] = 'primary'

        if 'datastore' in instance_info:
            if instance.datastore.get('version'):
                instance_info['datastore_version'] = instance.\
                    datastore['version']
            instance_info['datastore'] = instance.datastore['type']

        if 'access' in instance_info:
            instance_info['public'] = instance_info["access"].get(
                "is_public", False)

        if 'addresses' not in instance_info:
            instance_info['addresses'] = ''

        if 'operating_status' not in instance_info:
            # In case newer version python-troveclient is talking to older
            # version trove.
            instance_info['operating_status'] = ''

        instances_info.append(instance_info)

    return instances_info


def set_attributes_for_print_detail(instance):
    info = instance.to_dict()
    info['flavor'] = instance.flavor['id']
    if hasattr(instance, 'volume'):
        info['volume'] = instance.volume['size']
        if 'used' in instance.volume:
            info['volume_used'] = instance.volume['used']
    if hasattr(instance, 'ip'):
        info['ip'] = ', '.join(instance.ip)
    if hasattr(instance, 'datastore'):
        info['datastore'] = instance.datastore['type']
        info['datastore_version'] = instance.datastore['version']
        info['datastore_version_number'] = instance.datastore.get(
            'version_number')
    if hasattr(instance, 'configuration'):
        info['configuration'] = instance.configuration['id']
    if hasattr(instance, 'replica_of'):
        info['replica_of'] = instance.replica_of['id']
    if hasattr(instance, 'replicas'):
        replicas = [replica['id'] for replica in instance.replicas]
        info['replicas'] = ', '.join(replicas)
    if hasattr(instance, 'networks'):
        info['networks'] = instance.networks['name']
        info['networks_id'] = instance.networks['id']
    if hasattr(instance, 'fault'):
        info.pop('fault', None)
        info['fault'] = instance.fault['message']
        info['fault_date'] = instance.fault['created']
        if 'details' in instance.fault and instance.fault['details']:
            info['fault_details'] = instance.fault['details']
    if hasattr(instance, 'access'):
        info['public'] = instance.access.get("is_public", False)
        info['allowed_cidrs'] = instance.access.get('allowed_cidrs', [])
        info.pop("access", None)

    info.pop('links', None)
    return info


class ListDatabaseInstances(command.Lister):
    _description = _("List database instances")
    columns = ['ID', 'Name', 'Datastore', 'Datastore Version', 'Status',
               'Operating Status', 'Public', 'Addresses', 'Flavor ID',
               'Size', 'Role']
    admin_columns = columns + ["Server ID", "Tenant ID"]

    def get_parser(self, prog_name):
        parser = super(ListDatabaseInstances, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            default=None,
            help=_('Limit the number of results displayed.')
        )
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<ID>',
            type=str,
            default=None,
            help=_('Begin displaying the results for IDs greater than the'
                   'specified marker. When used with ``--limit``, set '
                   'this to the last ID displayed in the previous run.')
        )
        parser.add_argument(
            '--include_clustered', '--include-clustered',
            dest='include_clustered',
            action="store_true",
            default=False,
            help=_("Include instances that are part of a cluster "
                   "(default %(default)s).  --include-clustered may be "
                   "deprecated in the future, retaining just "
                   "--include_clustered.")
        )
        parser.add_argument(
            '--all-projects',
            dest='all_projects',
            action="store_true",
            default=False,
            help=_("Include database instances of all projects (admin only)")
        )
        parser.add_argument(
            '--project-id',
            help=_("Include database instances of a specific project "
                   "(admin only)")
        )
        return parser

    def take_action(self, parsed_args):
        extra_params = {}
        if parsed_args.all_projects or parsed_args.project_id:
            db_instances = self.app.client_manager.database.mgmt_instances
            cols = self.admin_columns
            if parsed_args.project_id:
                extra_params['project_id'] = parsed_args.project_id
        else:
            db_instances = self.app.client_manager.database.instances
            cols = self.columns

        instances = db_instances.list(
            limit=parsed_args.limit,
            marker=parsed_args.marker,
            include_clustered=parsed_args.include_clustered,
            **extra_params
        )
        if instances:
            instances_info = get_instances_info(instances)
            instances = [osc_utils.get_dict_properties(info, cols)
                         for info in instances_info]

        return cols, instances


class ShowDatabaseInstance(command.ShowOne):
    _description = _("Show instance details")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Instance (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances, parsed_args.instance)
        instance = set_attributes_for_print_detail(instance)
        return zip(*sorted(instance.items()))


class DeleteDatabaseInstance(base.TroveDeleter):
    _description = _("Deletes an instance.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            nargs='+',
            metavar='instance',
            help='Id or name of instance(s).'
        )
        parser.add_argument(
            '--force',
            action="store_true",
            default=False,
            help=_('Force delete the instance, will reset the instance status '
                   'before deleting.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances

        # Used for batch deletion
        self.delete_func = (db_instances.force_delete if parsed_args.force
                            else db_instances.delete)
        self.resource = 'database instance'

        ids = []
        for instance_id in parsed_args.instance:
            if not uuidutils.is_uuid_like(instance_id):
                try:
                    instance_id = trove_utils.get_resource_id_by_name(
                        db_instances, instance_id
                    )
                except Exception as e:
                    msg = ("Failed to get database instance %s, error: %s" %
                           (instance_id, str(e)))
                    raise exceptions.CommandError(msg)

            ids.append(instance_id)

        self.delete_resources(ids)


class CreateDatabaseInstance(command.ShowOne):
    _description = _("Creates a new database instance.")

    def get_parser(self, prog_name):
        parser = super(CreateDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_("Name of the instance."),
        )
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            type=str,
            help=_("Flavor to create the instance (name or ID). Flavor is not "
                   "required when creating replica instances."),
        )
        parser.add_argument(
            '--size',
            metavar='<size>',
            type=int,
            default=None,
            help=_("Size of the instance disk volume in GB. "
                   "Required when volume support is enabled."),
        )
        parser.add_argument(
            '--volume-type',
            metavar='<volume_type>',
            type=str,
            default=None,
            help=_("Volume type. Optional when volume support is enabled."),
        )
        parser.add_argument(
            '--databases',
            metavar='<database>',
            nargs="+",
            default=[],
            help=_("Optional list of databases."),
        )
        parser.add_argument(
            '--users',
            metavar='<user:password>',
            nargs="+",
            default=[],
            help=_("Optional list of users."),
        )
        parser.add_argument(
            '--backup',
            metavar='<backup>',
            default=None,
            help=_("A backup name or ID."),
        )
        parser.add_argument(
            '--availability-zone',
            metavar='<availability_zone>',
            default=None,
            help=_("The Zone hint to give to Nova."),
        )
        parser.add_argument(
            '--datastore',
            metavar='<datastore>',
            default=None,
            help=_("A datastore name or ID."),
        )
        parser.add_argument(
            '--datastore-version',
            metavar='<datastore_version>',
            default=None,
            help=_("A datastore version name or ID."),
        )
        parser.add_argument(
            '--datastore-version-number',
            default=None,
            help=_('The version number for the database. The version number '
                   'is needed for the datastore versions with the same name.'),
        )
        parser.add_argument(
            '--nic',
            metavar=('<net-id=<net-uuid>,subnet-id=<subnet-uuid>,'
                     'ip-address=<ip-address>>'),
            dest='nics',
            help=_("Create instance in the given Neutron network. This "
                   "information is used for creating user-facing port for the "
                   "instance. Either network ID or subnet ID (or both) should "
                   "be specified, IP address is optional"),
        )
        parser.add_argument(
            '--configuration',
            metavar='<configuration>',
            default=None,
            help=_("ID of the configuration group to attach to the instance."),
        )
        parser.add_argument(
            '--replica-of',
            metavar='<source_instance>',
            default=None,
            help=_("ID or name of an existing instance to replicate from."),
        )
        parser.add_argument(
            '--replica-count',
            metavar='<count>',
            type=int,
            default=None,
            help=_("Number of replicas to create (defaults to 1 if "
                   "replica_of specified)."),
        )
        parser.add_argument(
            '--module',
            metavar='<module>',
            type=str,
            dest='modules',
            action='append',
            default=[],
            help=_("ID or name of the module to apply.  Specify multiple "
                   "times to apply multiple modules."),
        )
        parser.add_argument(
            '--locality',
            metavar='<policy>',
            default=None,
            choices=['affinity', 'anti-affinity'],
            help=_("Locality policy to use when creating replicas. Choose "
                   "one of %(choices)s."),
        )
        parser.add_argument(
            '--region',
            metavar='<region>',
            type=str,
            default=None,
            help=argparse.SUPPRESS,
        )
        parser.add_argument(
            '--is-public',
            action='store_true',
            help="Whether or not to make the instance public.",
        )
        parser.add_argument(
            '--allowed-cidr',
            action='append',
            dest='allowed_cidrs',
            help="The IP CIDRs that are allowed to access the database "
                 "instance. Repeat for multiple values",
        )
        return parser

    def take_action(self, parsed_args):
        database = self.app.client_manager.database
        db_instances = database.instances

        if not parsed_args.replica_of and not parsed_args.flavor:
            raise exceptions.CommandError(_("Please specify a flavor"))

        if parsed_args.replica_of and parsed_args.flavor:
            print("Warning: Flavor is ignored for creating replica.")

        if not parsed_args.replica_of:
            flavor_id = osc_utils.find_resource(
                database.flavors, parsed_args.flavor).id
        else:
            flavor_id = None

        volume = None
        if parsed_args.size is not None and parsed_args.size <= 0:
            raise exceptions.ValidationError(
                _("Volume size '%s' must be an integer and greater than 0.")
                % parsed_args.size)
        elif parsed_args.size:
            volume = {"size": parsed_args.size,
                      "type": parsed_args.volume_type}
        restore_point = None
        if parsed_args.backup:
            restore_point = {"backupRef": osc_utils.find_resource(
                database.backups, parsed_args.backup).id}
        replica_of = None
        replica_count = parsed_args.replica_count
        if parsed_args.replica_of:
            replica_of = osc_utils.find_resource(
                db_instances, parsed_args.replica_of)
            replica_count = replica_count or 1
        locality = None
        if parsed_args.locality:
            locality = parsed_args.locality
            if replica_of:
                raise exceptions.ValidationError(
                    _('Cannot specify locality when adding replicas '
                      'to existing master.'))
        databases = [{'name': value} for value in parsed_args.databases]
        users = [{'name': n, 'password': p, 'databases': databases} for (n, p)
                 in
                 [z.split(':')[:2] for z in parsed_args.users]]

        nics = []
        if parsed_args.nics:
            nic_info = {}
            allowed_keys = {
                'net-id': 'network_id',
                'subnet-id': 'subnet_id',
                'ip-address': 'ip_address'
            }
            fields = parsed_args.nics.split(',')
            for field in fields:
                field = field.strip()
                k, v = field.split('=', 1)
                k = k.strip()
                v = v.strip()
                if k not in allowed_keys.keys():
                    raise exceptions.ValidationError(
                        f"{k} is not allowed."
                    )
                if v:
                    nic_info[allowed_keys[k]] = v
            nics.append(nic_info)

        modules = []
        for module in parsed_args.modules:
            modules.append(osc_utils.find_resource(database.modules,
                                                   module).id)

        access = {'is_public': False}
        if parsed_args.is_public:
            access['is_public'] = True
        if parsed_args.allowed_cidrs:
            access['allowed_cidrs'] = parsed_args.allowed_cidrs

        instance = db_instances.create(
            parsed_args.name,
            flavor_id=flavor_id,
            volume=volume,
            databases=databases,
            users=users,
            restorePoint=restore_point,
            availability_zone=(parsed_args.availability_zone),
            datastore=parsed_args.datastore,
            datastore_version=(parsed_args.datastore_version),
            datastore_version_number=(parsed_args.datastore_version_number),
            nics=nics,
            configuration=parsed_args.configuration,
            replica_of=replica_of,
            replica_count=replica_count,
            modules=modules,
            locality=locality,
            region_name=parsed_args.region,
            access=access
        )
        instance = set_attributes_for_print_detail(instance)
        return zip(*sorted(instance.items()))


class ResetDatabaseInstanceStatus(command.Command):
    _description = _("Set instance service status to ERROR and clear the "
                     "current task status. Mark any running backup operations "
                     "as FAILED.")

    def get_parser(self, prog_name):
        parser = super(ResetDatabaseInstanceStatus, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        db_instances.reset_status(instance)


class ResizeDatabaseInstanceFlavor(command.Command):
    _description = _("Resize an instance with a new flavor")

    def get_parser(self, prog_name):
        parser = super(ResizeDatabaseInstanceFlavor, self).get_parser(
            prog_name
        )
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance')
        )
        parser.add_argument(
            'flavor',
            type=str,
            help=_('ID or name of the new flavor.')
        )
        return parser

    def take_action(self, parsed_args):
        instance_mgr = self.app.client_manager.database.instances
        flavor_mgr = self.app.client_manager.database.flavors

        instance_id = parsed_args.instance
        if not uuidutils.is_uuid_like(instance_id):
            instance = osc_utils.find_resource(instance_mgr, instance_id)
            instance_id = instance.id

        flavor = osc_utils.find_resource(flavor_mgr, parsed_args.flavor)

        instance_mgr.resize_instance(instance_id, flavor.id)


class UpgradeDatabaseInstance(command.Command):
    _description = _("Upgrades an instance to a new datastore version.")

    def get_parser(self, prog_name):
        parser = super(UpgradeDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'),
        )
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('ID or name of the datastore version.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        db_instances.upgrade(instance, parsed_args.datastore_version)


class ResizeDatabaseInstanceVolume(command.Command):
    _description = _("Resizes the volume size of an instance.")

    def get_parser(self, prog_name):
        parser = super(ResizeDatabaseInstanceVolume, self).get_parser(
            prog_name
        )
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            'size',
            metavar='<size>',
            type=int,
            default=None,
            help=_('New size of the instance disk volume in GB.')
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        db_instances.resize_volume(instance, parsed_args.size)


class ForceDeleteDatabaseInstance(command.Command):
    _description = _("Force delete an instance.")

    def get_parser(self, prog_name):
        parser = (
            super(ForceDeleteDatabaseInstance, self).get_parser(prog_name))
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        db_instances.reset_status(instance)
        try:
            db_instances.delete(instance)
        except Exception as e:
            msg = (_("Failed to delete instance %(instance)s: %(e)s")
                   % {'instance': parsed_args.instance, 'e': e})
            raise exceptions.CommandError(msg)


class PromoteDatabaseInstanceToReplicaSource(command.Command):
    _description = _(
        "Promotes a replica to be the new replica source of its set.")

    def get_parser(self, prog_name):
        parser = super(PromoteDatabaseInstanceToReplicaSource,
                       self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        db_instances.promote_to_replica_source(instance)


class RestartDatabaseInstance(command.Command):
    _description = _("Restarts an instance.")

    def get_parser(self, prog_name):
        parser = super(RestartDatabaseInstance, self).get_parser(
            prog_name
        )
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.')
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        db_instances.restart(instance)


class EjectDatabaseInstanceReplicaSource(command.Command):
    _description = _("Ejects a replica source from its set.")

    def get_parser(self, prog_name):
        parser = super(EjectDatabaseInstanceReplicaSource, self).get_parser(
            prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        db_instances.eject_replica_source(instance)


class UpdateDatabaseInstance(command.Command):
    _description = _("Updates an instance: Edits name, "
                     "configuration, or replica source.")

    def get_parser(self, prog_name):
        parser = super(UpdateDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            type=str,
            default=None,
            help=_('ID or name of the instance.'),
        )
        parser.add_argument(
            '--configuration',
            metavar='<configuration>',
            type=str,
            default=None,
            help=_('ID of the configuration reference to attach.'),
        )
        parser.add_argument(
            '--detach_replica_source',
            '--detach-replica-source',
            dest='detach_replica_source',
            action="store_true",
            default=False,
            help=_('Detach the replica instance from its replication source. '
                   '--detach-replica-source may be deprecated in the future '
                   'in favor of just --detach_replica_source'),
        )
        parser.add_argument(
            '--remove_configuration',
            '--remove-configuration',
            dest='remove_configuration',
            action="store_true",
            default=False,
            help=_('Drops the current configuration reference.'),
        )
        public_group = parser.add_mutually_exclusive_group()
        public_group.add_argument(
            '--is-public',
            dest='public',
            default=None,
            action='store_true',
            help="Make the database instance accessible to public.",
        )
        public_group.add_argument(
            '--is-private',
            dest='public',
            default=None,
            action='store_false',
            help="Make the database instance inaccessible to public.",
        )
        parser.add_argument(
            '--allowed-cidr',
            action='append',
            dest='allowed_cidrs',
            help="The IP CIDRs that are allowed to access the database "
                 "instance. Repeat for multiple values",
        )
        return parser

    def take_action(self, parsed_args):
        instance_mgr = self.app.client_manager.database.instances
        instance_id = parsed_args.instance

        if not uuidutils.is_uuid_like(instance_id):
            instance_id = osc_utils.find_resource(instance_mgr, instance_id)

        instance_mgr.update(instance_id, parsed_args.configuration,
                            parsed_args.name,
                            parsed_args.detach_replica_source,
                            parsed_args.remove_configuration,
                            is_public=parsed_args.public,
                            allowed_cidrs=parsed_args.allowed_cidrs)


class DetachDatabaseInstanceReplica(command.Command):
    _description = _("Detaches a replica instance "
                     "from its replication source.")

    def get_parser(self, prog_name):
        parser = super(DetachDatabaseInstanceReplica, self).get_parser(
            prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        db_instances.update(instance, detach_replica_source=True)


class RebootDatabaseInstance(command.Command):
    _description = _("Reboots an instance(the Nova server).")

    def get_parser(self, prog_name):
        parser = super(RebootDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'))

        return parser

    def take_action(self, parsed_args):
        instance_id = parsed_args.instance

        if not uuidutils.is_uuid_like(instance_id):
            instance_mgr = self.app.client_manager.database.instances
            instance_id = osc_utils.find_resource(instance_mgr, instance_id)

        mgmt_instance_mgr = self.app.client_manager.database.mgmt_instances
        mgmt_instance_mgr.reboot(instance_id)


class RebuildDatabaseInstance(command.Command):
    _description = _("Rebuilds an instance(the Nova server).")

    def get_parser(self, prog_name):
        parser = super(RebuildDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'))
        parser.add_argument(
            'image',
            metavar='<image-id>',
            help=_('ID of the new guest image.'))

        return parser

    def take_action(self, parsed_args):
        instance_id = parsed_args.instance

        if not uuidutils.is_uuid_like(instance_id):
            instance_mgr = self.app.client_manager.database.instances
            instance_id = osc_utils.find_resource(instance_mgr, instance_id)

        mgmt_instance_mgr = self.app.client_manager.database.mgmt_instances
        mgmt_instance_mgr.rebuild(instance_id, parsed_args.image)



# Copyright 2020 Catalyst Cloud
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
from troveclient import base
from troveclient import common


class BackupStrategy(base.Resource):
    def __repr__(self):
        return "<BackupStrategy: %s[%s]>" % (self.project_id, self.instance_id)


class BackupStrategiesManager(base.ManagerWithFind):
    resource_class = BackupStrategy

    def list(self, instance_id=None, project_id=None):
        query_strings = {}
        if instance_id:
            query_strings["instance_id"] = instance_id
        if project_id:
            query_strings["project_id"] = project_id

        url = common.append_query_strings('/backup_strategies',
                                          **query_strings)

        return self._list(url, "backup_strategies")

    def create(self, instance_id=None, swift_container=None):
        backup_strategy = {}
        if instance_id:
            backup_strategy['instance_id'] = instance_id
        if swift_container:
            backup_strategy['swift_container'] = swift_container
        body = {"backup_strategy": backup_strategy}

        return self._create("/backup_strategies", body, "backup_strategy")

    def delete(self, instance_id=None, project_id=None):
        url = "/backup_strategies"
        query_strings = {}
        if instance_id:
            query_strings["instance_id"] = instance_id
        if project_id:
            query_strings["project_id"] = project_id

        url = common.append_query_strings('/backup_strategies',
                                          **query_strings)

        resp, body = self._delete(url)
        common.check_for_exceptions(resp, body, url)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import json

from troveclient.apiclient import exceptions
from troveclient.tests import utils as test_utils


class ExceptionsTestCase(test_utils.TestCase):

    def _test_from_response(self, body):
        data = {
            'status_code': 503,
            'headers': {
                'Content-Type': 'application/json',
                'x-compute-request-id': (
                    'req-65d6443c-5910-4eb4-b48a-e69849c26836'),
            },
            'text': json.dumps(body)
        }
        response = test_utils.TestResponse(data)
        fake_url = 'http://localhost:8779/v1.0/fake/instances'
        error = exceptions.from_response(response, 'GET', fake_url)
        self.assertIsInstance(error, exceptions.ServiceUnavailable)

    def test_from_response_webob_pre_1_6_0(self):
        # Tests error responses before webob 1.6.0 where the error details
        # are nested in the response body.
        body = {
            'serviceUnavailable': {
                'message': 'Fake message.',
                'code': 503
            }
        }
        self._test_from_response(body)

    def test_from_response_webob_post_1_6_0(self):
        # Tests error responses from webob 1.6.0 where the error details
        # are in the response body.
        body = {'message': 'Fake message.', 'code': 503}
        self._test_from_response(body)



#!/usr/bin/env python

#    Copyright 2011 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Trove Management Command line tool
"""

import json
import os
import sys

from troveclient.compat import common

# If ../trove/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                                os.pardir,
                                                os.pardir))
if os.path.exists(os.path.join(possible_topdir, 'troveclient.compat',
                               '__init__.py')):
    sys.path.insert(0, possible_topdir)


oparser = None


def _pretty_print(info):
    print(json.dumps(info, sort_keys=True, indent=4))


class HostCommands(common.AuthedCommandsBase):
    """Commands to list info on hosts."""

    params = [
        'name',
    ]

    def update_all(self):
        """Update all instances on a host."""
        self._require('name')
        self.dbaas.hosts.update_all(self.name)

    def get(self):
        """List details for the specified host."""
        self._require('name')
        self._pretty_print(self.dbaas.hosts.get, self.name)

    def list(self):
        """List all compute hosts."""
        self._pretty_list(self.dbaas.hosts.index)


class QuotaCommands(common.AuthedCommandsBase):
    """List and update quota limits for a tenant."""

    params = ['id',
              'instances',
              'volumes',
              'backups']

    def list(self):
        """List all quotas for a tenant."""
        self._require('id')
        self._pretty_print(self.dbaas.quota.show, self.id)

    def update(self):
        """Update quota limits for a tenant."""
        self._require('id')
        self._pretty_print(self.dbaas.quota.update, self.id,
                           dict((param, getattr(self, param))
                                for param in self.params if param != 'id'))


class RootCommands(common.AuthedCommandsBase):
    """List details about the root info for an instance."""

    params = [
        'id',
    ]

    def history(self):
        """List root history for the instance."""
        self._require('id')
        self._pretty_print(self.dbaas.management.root_enabled_history, self.id)


class AccountCommands(common.AuthedCommandsBase):
    """Commands to list account info."""

    params = [
        'id',
    ]

    def list(self):
        """List all accounts with non-deleted instances."""
        self._pretty_print(self.dbaas.accounts.index)

    def get(self):
        """List details for the account provided."""
        self._require('id')
        self._pretty_print(self.dbaas.accounts.show, self.id)


class InstanceCommands(common.AuthedCommandsBase):
    """List details about an instance."""

    params = [
        'deleted',
        'id',
        'limit',
        'marker',
        'host',
    ]

    def get(self):
        """List details for the instance."""
        self._require('id')
        self._pretty_print(self.dbaas.management.show, self.id)

    def list(self):
        """List all instances for account."""
        deleted = None
        if self.deleted is not None:
            if self.deleted.lower() in ['true']:
                deleted = True
            elif self.deleted.lower() in ['false']:
                deleted = False
        self._pretty_paged(self.dbaas.management.index, deleted=deleted)

    def hwinfo(self):
        """Show hardware information details about an instance."""
        self._require('id')
        self._pretty_print(self.dbaas.hwinfo.get, self.id)

    def diagnostic(self):
        """List diagnostic details about an instance."""
        self._require('id')
        self._pretty_print(self.dbaas.diagnostics.get, self.id)

    def stop(self):
        """Stop MySQL on the given instance."""
        self._require('id')
        self._pretty_print(self.dbaas.management.stop, self.id)

    def reboot(self):
        """Reboot the instance."""
        self._require('id')
        self._pretty_print(self.dbaas.management.reboot, self.id)

    def migrate(self):
        """Migrate the instance."""
        self._require('id')
        self._pretty_print(self.dbaas.management.migrate, self.id, self.host)

    def reset_task_status(self):
        """Set the instance's task status to NONE."""
        self._require('id')
        self._pretty_print(self.dbaas.management.reset_task_status, self.id)


class StorageCommands(common.AuthedCommandsBase):
    """Commands to list devices info."""

    params = []

    def list(self):
        """List details for the storage device."""
        self._pretty_list(self.dbaas.storage.index)


class FlavorsCommands(common.AuthedCommandsBase):
    """Commands for managing Flavors."""

    params = [
        'name',
        'ram',
        'disk',
        'vcpus',
        'flavor_id',
        'ephemeral',
        'swap',
        'rxtx_factor',
        'service_type'
    ]

    def create(self):
        """Create a new flavor."""
        self._require('name', 'ram', 'disk', 'vcpus',
                      'flavor_id', 'service_type')
        self._pretty_print(self.dbaas.mgmt_flavor.create, self.name,
                           self.ram, self.disk, self.vcpus, self.flavor_id,
                           self.ephemeral, self.swap, self.rxtx_factor,
                           self.service_type)


def config_options(oparser):
    oparser.add_option("-u", "--url", default="http://localhost:5000/v1.1",
                       help="Auth API endpoint URL with port and version. \
                            Default: http://localhost:5000/v1.1")


COMMANDS = {
    'account': AccountCommands,
    'host': HostCommands,
    'instance': InstanceCommands,
    'root': RootCommands,
    'storage': StorageCommands,
    'quota': QuotaCommands,
    'flavor': FlavorsCommands,
}


def main():
    # Parse arguments
    oparser = common.CliOptions.create_optparser(True)
    for k, v in COMMANDS.items():
        v._prepare_parser(oparser)
    (options, args) = oparser.parse_args()

    if not args:
        common.print_commands(COMMANDS)

    # Pop the command and check if it's in the known commands
    cmd = args.pop(0)
    if cmd in COMMANDS:
        fn = COMMANDS.get(cmd)
        command_object = None
        try:
            command_object = fn(oparser)
        except Exception as ex:
            if options.debug:
                raise
            print(ex)

        # Get a list of supported actions for the command
        actions = common.methods_of(command_object)

        if len(args) < 1:
            common.print_actions(cmd, actions)

        # Check for a valid action and perform that action
        action = args.pop(0)
        if action in actions:
            try:
                getattr(command_object, action)()
            except Exception as ex:
                if options.debug:
                    raise
                print(ex)
        else:
            common.print_actions(cmd, actions)
    else:
        common.print_commands(COMMANDS)


if __name__ == '__main__':
    main()



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Exception definitions
"""

# alias exceptions from apiclient for users of this module
from troveclient.apiclient.exceptions import *  # noqa


class NoTokenLookupException(Exception):
    """This form of authentication does not support looking up
       endpoints from an existing token.
    """
    pass


class ResponseFormatError(Exception):
    """Could not parse the response format."""
    pass


class GuestLogNotFoundError(Exception):
    """The specified guest log does not exist."""
    pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Database v1 Limits action implementations"""

from osc_lib.command import command
from osc_lib import utils as osc_utils

from troveclient.i18n import _
from troveclient import utils


class ListDatabaseLimits(command.Lister):

    _description = _("List database limits")
    columns = ['Value', 'Verb', 'Remaining', 'Unit']

    def take_action(self, parsed_args):
        database_limits = self.app.client_manager.database.limits
        limits = database_limits.list()
        # Pop the first one, its absolute limits
        utils.print_dict(limits.pop(0)._info)
        limits = [osc_utils.get_item_properties(i, self.columns)
                  for i in limits]
        return self.columns, limits



# Copyright (c) 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
import json
import optparse
import sys
from unittest import mock

import testtools

from troveclient.compat import common

"""
 unit tests for common.py
"""


class CommonTest(testtools.TestCase):

    def setUp(self):
        super(CommonTest, self).setUp()
        self.orig_sys_exit = sys.exit
        sys.exit = mock.Mock(return_value=None)

    def tearDown(self):
        super(CommonTest, self).tearDown()
        sys.exit = self.orig_sys_exit

    def test_methods_of(self):
        class DummyClass(object):
            def dummyMethod(self):
                print("just for test")

        obj = DummyClass()
        result = common.methods_of(obj)
        self.assertEqual(1, len(result))
        method = result['dummyMethod']
        self.assertIsNotNone(method)

    def test_check_for_exceptions(self):
        status = [400, 422, 500]
        for s in status:
            resp = mock.Mock()
            # compat still uses status
            resp.status = s
            self.assertRaises(Exception,
                              common.check_for_exceptions, resp, "body")

        # a no-exception case
        resp = mock.Mock()
        resp.status_code = 200
        common.check_for_exceptions(resp, "body")

    def test_print_actions(self):
        cmd = "test-cmd"
        actions = {"test": "test action", "help": "help action"}
        common.print_actions(cmd, actions)
        pass

    def test_print_commands(self):
        commands = {"cmd-1": "cmd 1", "cmd-2": "cmd 2"}
        common.print_commands(commands)
        pass

    def test_limit_url(self):
        url = "test-url"
        limit = None
        marker = None
        self.assertEqual(url, common.limit_url(url))

        limit = "test-limit"
        marker = "test-marker"
        expected = "test-url?marker=test-marker&limit=test-limit"
        self.assertEqual(expected,
                         common.limit_url(url, limit=limit, marker=marker))


class CliOptionsTest(testtools.TestCase):

    def check_default_options(self, co):
        self.assertIsNone(co.username)
        self.assertIsNone(co.apikey)
        self.assertIsNone(co.tenant_id)
        self.assertIsNone(co.auth_url)
        self.assertEqual('keystone', co.auth_type)
        self.assertEqual('database', co.service_type)
        self.assertEqual('RegionOne', co.region)
        self.assertIsNone(co.service_url)
        self.assertFalse(co.insecure)
        self.assertFalse(co.verbose)
        self.assertFalse(co.debug)
        self.assertIsNone(co.token)

    def check_option(self, oparser, option_name):
        option = oparser.get_option("--%s" % option_name)
        self.assertIsNotNone(option)
        if option_name in common.CliOptions.DEFAULT_VALUES:
            self.assertEqual(common.CliOptions.DEFAULT_VALUES[option_name],
                             option.default)

    def test___init__(self):
        co = common.CliOptions()
        self.check_default_options(co)

    def test_default(self):
        co = common.CliOptions.default()
        self.check_default_options(co)

    def test_load_from_file(self):
        co = common.CliOptions.load_from_file()
        self.check_default_options(co)

    def test_create_optparser(self):
        option_names = ["verbose", "debug", "auth_url", "username", "apikey",
                        "tenant_id", "auth_type", "service_type",
                        "service_name", "service_type", "service_name",
                        "service_url", "region", "insecure", "token",
                        "secure", "json", "terse", "hide-debug"]

        oparser = common.CliOptions.create_optparser(True)
        for option_name in option_names:
            self.check_option(oparser, option_name)

        oparser = common.CliOptions.create_optparser(False)
        for option_name in option_names:
            self.check_option(oparser, option_name)


class ArgumentRequiredTest(testtools.TestCase):

    def setUp(self):
        super(ArgumentRequiredTest, self).setUp()
        self.param = "test-param"
        self.arg_req = common.ArgumentRequired(self.param)

    def test___init__(self):
        self.assertEqual(self.param, self.arg_req.param)

    def test___str__(self):
        expected = 'Argument "--%s" required.' % self.param
        self.assertEqual(expected, self.arg_req.__str__())


class CommandsBaseTest(testtools.TestCase):

    def setUp(self):
        super(CommandsBaseTest, self).setUp()
        self.orig_sys_exit = sys.exit
        sys.exit = mock.Mock(return_value=None)
        self.orig_sys_argv = sys.argv
        sys.argv = ['fakecmd']
        parser = common.CliOptions().create_optparser(False)
        self.cmd_base = common.CommandsBase(parser)

    def tearDown(self):
        super(CommandsBaseTest, self).tearDown()
        sys.exit = self.orig_sys_exit
        sys.argv = self.orig_sys_argv

    def test___init__(self):
        self.assertIsNotNone(self.cmd_base)

    def test__safe_exec(self):
        func = mock.Mock(return_value="test")
        self.cmd_base.debug = True
        r = self.cmd_base._safe_exec(func)
        self.assertEqual("test", r)

        self.cmd_base.debug = False
        r = self.cmd_base._safe_exec(func)
        self.assertEqual("test", r)

        func = mock.Mock(side_effect=ValueError)  # an arbitrary exception
        r = self.cmd_base._safe_exec(func)
        self.assertIsNone(r)

    def test__prepare_parser(self):
        parser = optparse.OptionParser()
        common.CommandsBase.params = ["test_1", "test_2"]
        self.cmd_base._prepare_parser(parser)
        option = parser.get_option("--%s" % "test_1")
        self.assertIsNotNone(option)
        option = parser.get_option("--%s" % "test_2")
        self.assertIsNotNone(option)

    def test__parse_options(self):
        parser = optparse.OptionParser()
        parser.add_option("--%s" % "test_1", default="test_1v")
        parser.add_option("--%s" % "test_2", default="test_2v")
        self.cmd_base._parse_options(parser)
        self.assertEqual("test_1v", self.cmd_base.test_1)
        self.assertEqual("test_2v", self.cmd_base.test_2)

    def test__require(self):
        self.assertRaises(common.ArgumentRequired,
                          self.cmd_base._require, "attr_1")
        self.cmd_base.attr_1 = None
        self.assertRaises(common.ArgumentRequired,
                          self.cmd_base._require, "attr_1")
        self.cmd_base.attr_1 = "attr_v1"
        self.cmd_base._require("attr_1")

    def test__make_list(self):
        self.assertRaises(AttributeError, self.cmd_base._make_list, "attr1")
        self.cmd_base.attr1 = "v1,v2"
        self.cmd_base._make_list("attr1")
        self.assertEqual(["v1", "v2"], self.cmd_base.attr1)
        self.cmd_base.attr1 = ["v3"]
        self.cmd_base._make_list("attr1")
        self.assertEqual(["v3"], self.cmd_base.attr1)

    def test__pretty_print(self):
        func = mock.Mock(return_value=None)
        self.cmd_base.verbose = True
        self.assertIsNone(self.cmd_base._pretty_print(func))
        self.cmd_base.verbose = False
        self.assertIsNone(self.cmd_base._pretty_print(func))

    def test__dumps(self):
        orig_dumps = json.dumps
        json.dumps = mock.Mock(return_value="test-dump")
        self.assertEqual("test-dump", self.cmd_base._dumps("item"))
        json.dumps = orig_dumps

    def test__pretty_list(self):
        func = mock.Mock(return_value=None)
        self.cmd_base.verbose = True
        self.assertIsNone(self.cmd_base._pretty_list(func))
        self.cmd_base.verbose = False
        self.assertIsNone(self.cmd_base._pretty_list(func))
        item = mock.Mock(return_value="test")
        item._info = "info"
        func = mock.Mock(return_value=[item])
        self.assertIsNone(self.cmd_base._pretty_list(func))

    def test__pretty_paged(self):
        self.cmd_base.limit = "5"
        func = mock.Mock(return_value=None)
        self.cmd_base.verbose = True
        self.assertIsNone(self.cmd_base._pretty_paged(func))

        self.cmd_base.verbose = False

        class MockIterable(collections.abc.Iterable):
            links = ["item"]
            count = 1

            def __iter__(self):
                return ["item1"]

            def __len__(self):
                return self.count

        ret = MockIterable()
        func = mock.Mock(return_value=ret)
        self.assertIsNone(self.cmd_base._pretty_paged(func))

        ret.count = 0
        self.assertIsNone(self.cmd_base._pretty_paged(func))

        func = mock.Mock(side_effect=ValueError)
        self.assertIsNone(self.cmd_base._pretty_paged(func))
        self.cmd_base.debug = True
        self.cmd_base.marker = mock.Mock()
        self.assertRaises(ValueError, self.cmd_base._pretty_paged, func)


class AuthTest(testtools.TestCase):

    def setUp(self):
        super(AuthTest, self).setUp()
        self.orig_sys_exit = sys.exit
        sys.exit = mock.Mock(return_value=None)
        self.orig_sys_argv = sys.argv
        sys.argv = ['fakecmd']
        self.parser = common.CliOptions().create_optparser(False)
        self.auth = common.Auth(self.parser)

    def tearDown(self):
        super(AuthTest, self).tearDown()
        sys.exit = self.orig_sys_exit
        sys.argv = self.orig_sys_argv

    def test___init__(self):
        self.assertIsNone(self.auth.dbaas)
        self.assertIsNone(self.auth.apikey)

    def test_login(self):
        self.auth.username = "username"
        self.auth.apikey = "apikey"
        self.auth.tenant_id = "tenant_id"
        self.auth.auth_url = "auth_url"
        dbaas = mock.Mock()
        dbaas.authenticate = mock.Mock(return_value=None)
        dbaas.client = mock.Mock()
        dbaas.client.auth_token = mock.Mock()
        dbaas.client.service_url = mock.Mock()
        self.auth._get_client = mock.Mock(return_value=dbaas)
        self.auth.login()

        self.auth.debug = True
        self.auth._get_client = mock.Mock(side_effect=ValueError)
        self.assertRaises(ValueError, self.auth.login)

        self.auth.debug = False
        self.auth.login()


class AuthedCommandsBaseTest(testtools.TestCase):

    def setUp(self):
        super(AuthedCommandsBaseTest, self).setUp()
        self.orig_sys_exit = sys.exit
        sys.exit = mock.Mock(return_value=None)
        self.orig_sys_argv = sys.argv
        sys.argv = ['fakecmd']

    def tearDown(self):
        super(AuthedCommandsBaseTest, self).tearDown()
        sys.exit = self.orig_sys_exit
        self.orig_sys_argv = sys.argv

    def test___init__(self):
        parser = common.CliOptions().create_optparser(False)
        common.AuthedCommandsBase.debug = True
        dbaas = mock.Mock()
        dbaas.authenticate = mock.Mock(return_value=None)
        dbaas.client = mock.Mock()
        dbaas.client.auth_token = mock.Mock()
        dbaas.client.service_url = mock.Mock()
        dbaas.client.authenticate_with_token = mock.Mock()
        common.AuthedCommandsBase._get_client = mock.Mock(return_value=dbaas)
        common.AuthedCommandsBase(parser)


class PaginatedTest(testtools.TestCase):

    def setUp(self):
        super(PaginatedTest, self).setUp()
        self.items_ = ["item1", "item2"]
        self.next_marker_ = "next-marker"
        self.links_ = ["link1", "link2"]
        self.pgn = common.Paginated(self.items_, self.next_marker_,
                                    self.links_)

    def tearDown(self):
        super(PaginatedTest, self).tearDown()

    def test___init__(self):
        self.assertEqual(self.items_, self.pgn.items)
        self.assertEqual(self.next_marker_, self.pgn.next)
        self.assertEqual(self.links_, self.pgn.links)

    def test___len__(self):
        self.assertEqual(len(self.items_), self.pgn.__len__())

    def test___iter__(self):
        itr_expected = self.items_.__iter__()
        itr = self.pgn.__iter__()
        self.assertEqual(next(itr_expected), next(itr))
        self.assertEqual(next(itr_expected), next(itr))
        self.assertRaises(StopIteration, next, itr_expected)
        self.assertRaises(StopIteration, next, itr)

    def test___getitem__(self):
        self.assertEqual(self.items_[0], self.pgn.__getitem__(0))

    def test___setitem__(self):
        self.pgn.__setitem__(0, "new-item")
        self.assertEqual("new-item", self.pgn.items[0])

    def test___delitem(self):
        del self.pgn[0]
        self.assertEqual(1, self.pgn.__len__())

    def test___reversed__(self):
        itr = self.pgn.__reversed__()
        self.assertEqual("item2", next(itr))
        self.assertEqual("item1", next(itr))
        self.assertRaises(StopIteration, next, itr)

    def test___contains__(self):
        self.assertTrue(self.pgn.__contains__("item1"))
        self.assertTrue(self.pgn.__contains__("item2"))
        self.assertFalse(self.pgn.__contains__("item3"))
